//=============================================================================
// Pinata remixed for Monster Hunt
// Remix by gopostal
// Original Pinata:
// Author: |404|shiftre
// Pawn drops all inventory, powerups only, or weapons only when killed
//=============================================================================
class MHMPinata expands Mutator;

var bool noWeapons;
var bool noPowerups;
var bool HealthDrop;
var bool bInitialized;

function PostBeginPlay()
{
	if ( bInitialized )
		return;

	noWeapons = class'PinataClientWindow'.Default.noWeapons;
	noPowerups = class'PinataClientWindow'.Default.noPowerups;
	HealthDrop = class'PinataClientWindow'.Default.HealthDrop;

	bInitialized = TRUE;
}

	
function bool PreventDeath (Pawn Killed, Pawn Killer, name damageType, vector HitLocation)
{
	local actor dropped;
	local float speed;
	local float temp;
	local int charge;
	local int finalcount;
	local int timecharge;
	local bool firstpass;
	local vector X,Y,Z;

	//Local list of droppable inv types
	local inventory inv;
	local weapon weap;
	local udamage udam;
	local ut_jumpboots jboots; //No, not those EverCamped items ;)

	firstpass = true;

	  // This check only triggers on a monster caused death. It keeps piles of weapons from gathering under the tougher
	  // jumping sections and difficult map spots
	  if(Killer != None && Killer.IsA('ScriptedPawn'))
	  {
	//Cycle through inventory, dropping weapons, pickups and armor
	for (inv=Killed.Inventory; inv!=None; inv=inv.Inventory)
	{
		weap = Weapon(inv);
		if ( (weap != None))
		{		
			//If noWeapons selected, toss out selected item only (if allowable)	
			if (noWeapons) 
                        {
				if (firstpass) 
                                {
                                        // These next couple of lines will remove the deemer from a pinata. I liked having it in the dropped pile so I
                                        // nulled it. If you don't want the deemer, add the lines back in
                                        /*
					if (WarheadLauncher(Other.Weapon)!=none && Other.Weapon.AmmoType.AmmoAmount == 0)
                                        {
					  Other.Inventory.Destroy();
					  continue;
				        }
					*/

					 if (ImpactHammer(Killed.Weapon)==None && ChainSaw(Killed.Weapon)==None && Killed.Weapon.bCanThrow) 
                                         {
						Killed.TossWeapon();
					 }
					firstpass = false;
				}								
				continue;
			}			
			
			//Get ammo count for this weapon			
			charge = weap.AmmoType.AmmoAmount;
			
			if (WarheadLauncher(weap)!=none && charge == 0)
				continue;	
				
			//Do not drop if impact hammer or chainsaw
			if (ImpactHammer(weap)!= None || ChainSaw(weap)!=none || !weap.bCanThrow) 
				continue;																				

			//Spawn this item & set attributes
			dropped = Spawn(weap.Class,,,Killed.Location);
			weap = Weapon(dropped);			
			
						
			if (charge ==0)
				weap.PickupAmmoCount=1;
			else
				weap.PickupAmmoCount = charge;
			if ( weap != None )
			{ 
				weap.RespawnTime = 0.0; //don't respawn
				weap.BecomePickup();
				weap.bTossedOut = true;	
				weap.bWeaponStay = false;
				weap.GotoState('PickUp', 'Dropped');	
			}
			
		}
		else if (!noPowerUps) 
                {
			 if (UDamage(inv) != None) 
                         {
				udam = UDamage(inv);
				if (udam != None) 
                                {
					//don't make much sense, but works										
					if (udam.TimerRate-udam.TimerCounter < 5)
						charge = 50;
					else 
						charge =((udam.TimerRate-udam.TimerCounter)*10);
									
					dropped = Spawn(udam.Class,,,Killed.Location);
					udam = UDamage(dropped);
					udam.Charge = charge;				
					if (udam != None) 
                                        {
						udam.RespawnTime = 0.0;
						udam.BecomePickup();
					}
				}	
			} 							
			else if (UT_jumpboots(inv) != None) 
                        {
				jboots = UT_jumpboots(inv);
				if (jboots != None) {				
					//Since jboots have a timelimit & charge limit, record both				
					timecharge = jboots.TimeCharge;
					charge = jboots.Charge;
					dropped = Spawn(jboots.Class,,,Killed.Location);
					jboots = UT_jumpboots(dropped);
					jboots.TimeCharge = timecharge;
					jboots.Charge = charge;
					if (jboots != None) 
                                        {
						jboots.RespawnTime = 0.0;
						jboots.BecomePickup();
					}
				}				
			}	//Else drop anything else that is not ammo (dropped with weapons) 
			else if (Ammo(inv) == None) 
                        {
				charge = inv.Charge;
				dropped = Spawn(inv.Class,,,Killed.Location);
				inv = Inventory(dropped);
				inv.Charge = charge;
				if (inv != None) 
                                {
					inv.RespawnTime = 0.0;
					inv.BecomePickup();
				}
			}			
		}				
				
		//Now set the speed of the dropped item based off victom's speed
		setVelocity (Killed, dropped);

		//Destroy Item from victom's inventory	
		Killed.Inventory.Destroy();
	}
	
	if (HealthDrop)
		dropRandomHealth (Killed);
	
	//Call next mutator down the line
	if ( NextMutator != None )
		NextMutator.ScoreKill(Killer, Killed);
        }
}

function dropRandomHealth (Pawn Other)
{
	local int   lottery;
	local int   i;
	local actor dropped;
	local float speed;
	
	lottery = Rand(100);
	
	BroadcastMessage ("Random ->"@lottery);
	
	if (lottery<=25);		
	else if (lottery<=55) {
		for (i=0; i<2; i++) {
			dropped = Spawn(class'HealthVial',,,Other.Location);
			setVelocity (Other, dropped);
			Inventory(dropped).RespawnTime = 0.0;
			Inventory(dropped).BecomePickup();		
		}
	}
	else if (lottery<=70) {
		dropped = Spawn(class'MedBox',,,Other.Location);
		setVelocity (Other, dropped);
		Inventory(dropped).RespawnTime = 0.0;
		Inventory(dropped).BecomePickup();	
	}
	else if (lottery<=85) {
		for (i=0; i<5; i++) {
			dropped = Spawn(class'HealthVial',,,Other.Location);
			setVelocity (Other, dropped);
			Inventory(dropped).RespawnTime = 0.0;
			Inventory(dropped).BecomePickup();	
		}
	}
	else if (lottery<=98) {
		for (i=0; i<1; i++) {
			dropped = Spawn(class'MedBox',,,Other.Location);
			setVelocity (Other, dropped);
			Inventory(dropped).RespawnTime = 0.0;
			Inventory(dropped).BecomePickup();	
		}		
	}
	else if (lottery<=100) {
		dropped = Spawn(class'HealthPack',,,Other.Location);
		setVelocity (Other, dropped);
		Inventory(dropped).RespawnTime = 0.0;
		Inventory(dropped).BecomePickup();	
	}
}


//Set's the velocity of an actor dropped by pawn Other
function setVelocity (Pawn Other, Actor dropped)		 
{
	local float speed;
	
	//Now set the speed of the dropped item based off victom's speed
	speed = VSize(Other.Velocity);			
	if ( dropped != None )	{
		dropped.RemoteRole = ROLE_DumbProxy;
		dropped.SetPhysics(PHYS_Falling);
		dropped.bCollideWorld = true;
		if (speed != 0)
			dropped.Velocity = Normal(Other.Velocity/speed + 0.5 * VRand()) * (speed + 280);
			else {
				dropped.Velocity.X = 0;
				dropped.Velocity.Y = 0;
				dropped.Velocity.Z = 0;
			}
	}	
}	

defaultproperties
{
     HealthDrop=True
}
